@@ -6,6 +6,7 @@ import pandas as pd
 import numpy as np
 import re
 from sktime.performance_metrics.forecasting import MeanAbsoluteScaledError
+from sklearn.metrics import mean_squared_error
 
 
 class ResultsHandler:
@@ -62,6 +63,10 @@ class ResultsHandler:
             self.algorithms_metadata[algorithm][
                 "version"
             ] = self._get_latest_version_algo(algorithm)
+            if not self.algorithms_metadata[algorithm]["version"]:
+                raise ValueError(
+                    f"Please make sure that you have result files for the {algorithm} algorithm"
+                )
 
     @staticmethod
     def _extract_version(filename):
@@ -94,28 +99,25 @@ class ResultsHandler:
         if param not in valid_values:
             raise ValueError(f"{param} is not a valid value")
 
-    def load_all_results_algo_list(
-        self, algorithms_list: List, res_type: str, res_measure: str
-    ) -> Tuple[List, List]:
-        results = []
-        algorithms = []
-        for algorithm in algorithms_list:
-            res, algorithm = self.load_results_algorithm(
-                algorithm=algorithm,
-                res_type=res_type,
-                res_measure=res_measure,
-                output_type="metrics",
+    def compute_error_metrics(self, metric: str = "mase") -> Dict:
+        metric_algorithm = {}
+        for algorithm in self.algorithms:
+            (
+                results_hierarchy,
+                results_by_group_element,
+                group_elements,
+            ) = self.compute_results_hierarchy(algorithm=algorithm)
+
+            metric_algorithm[algorithm] = self._compute_metric(
+                results_hierarchy,
+                results_by_group_element,
+                group_elements,
+                metric=metric,
             )
-            results.append(res)
-            algorithms.append(algorithm)
-
-        results = self._filter_list(results)
-        algorithms = self._filter_list(algorithms)
-
-        return results, algorithms
+        return metric_algorithm
 
     def load_results_algorithm(
-        self, algorithm: str, res_type: str, res_measure: str, output_type: str
+        self, algorithm: str, res_type: str, res_measure: str
     ) -> Tuple[Dict, str]:
         """
         Load results for a given algorithm.
@@ -125,56 +127,45 @@ class ResultsHandler:
             res_type: defines the type of results, could be 'fit_pred' to receive fitted values plus
                 predictions or 'pred' to only store predictions
             res_measure: defines the measure to store, could be 'mean' or 'std'
-            output_type: defines the type of output - 'results' or 'metrics'
 
         Returns:
             A list of results for the given algorithm.
         """
         algorithm_w_type = ""
         result = dict()
-        if algorithm in self.algorithms_metadata.keys():
-            if output_type == "metrics":
-                for file in [
-                    path
-                    for path in os.listdir(
-                        f"{self.path}{self.algorithms_metadata[algorithm]['algo_path']}"
-                    )
-                    if self.dataset in path
-                    and "orig" in path
-                    and self.algorithms_metadata[algorithm]["version"] in path
-                    and output_type in path
-                ]:
-                    result, algorithm_w_type = self._load_procedure(file, algorithm)
-            else:
-                for file in [
-                    path
-                    for path in os.listdir(
-                        f"{self.path}{self.algorithms_metadata[algorithm]['algo_path']}"
-                    )
-                    if self.dataset in path
-                    and "orig" in path
-                    and self.algorithms_metadata[algorithm]["version"] in path
-                    and res_type in path
-                    and res_measure in path
-                    and output_type in path
-                ]:
-                    result, algorithm_w_type = self._load_procedure(file, algorithm)
-
-        return result, algorithm_w_type
-
-    def _load_procedure(self, file, algorithm):
         # get the gp_type and concatenate with gpf_
-        match = re.search(r"gp_(.*)_cov", file)
+        match = re.search(r"gpf_(.*)", algorithm)
         if match:
             algo_type = match.group(1)
         else:
             algo_type = ""
 
+        if algorithm in self.algorithms_metadata.keys():
+            for file in [
+                path
+                for path in os.listdir(
+                    f"{self.path}{self.algorithms_metadata[algorithm]['algo_path']}"
+                )
+                if self.dataset in path
+                and "orig" in path
+                and self.algorithms_metadata[algorithm]["version"] in path
+                and res_type in path
+                and res_measure in path
+                and 'results' in path
+                and algo_type in path
+            ]:
+                result, algorithm_w_type = self._load_procedure(
+                    file, algorithm, algo_type
+                )
+
+        return result, algorithm_w_type
+
+    def _load_procedure(self, file, algorithm, algo_type):
         if (self.algorithms_metadata[algorithm]["preselected_algo_type"] != "") & (
             self.algorithms_metadata[algorithm]["preselected_algo_type"] == algo_type
         ):
             with open(
-                f"{self.path}/{self.algorithms_metadata[algorithm]['algo_path']}/{file}",
+                f"{self.path}{self.algorithms_metadata[algorithm]['algo_path']}/{file}",
                 "rb",
             ) as handle:
                 result = pickle.load(handle)
@@ -194,6 +185,49 @@ class ResultsHandler:
 
         return result, algorithm_w_type
 
+    def load_hyperparameters_logs(self, algorithm, path_to_logs="./logs/s"):
+        hyperparameters_dataset_algo = {}
+        for file_name in [
+            path
+            for path in os.listdir(f"{path_to_logs}")
+            if self.dataset in path
+            and self.algorithms_metadata[algorithm]["version"] in path
+            and self.algorithms_metadata[algorithm]["algo_path"] in path
+            and "hypertuning" in path
+        ]:
+            best_hyperparameters = {}
+            execution_date = ""
+            correct_log_structure = False
+            with open(f"{path_to_logs}{file_name}", "r") as f:
+                for line in f:
+                    if "Algorithm" in line:
+                        correct_log_structure = True
+                        algorithm = re.search(r"Algorithm: (.*?),", line).group(1)
+                        version = re.search(r"Version: (.*?),", line).group(1)
+                        dataset = re.search(r"Dataset: (.*?),", line).group(1)
+                        match = re.search(
+                            r"learning rate = (.*?), weight decay = (.*?), scheduler = (.*?), gamma = (.*?), inducing points percentage = (.*?), patience = (.*?)$",
+                            line,
+                        )
+                        best_hyperparameters = {
+                            "learning rate": match.group(1),
+                            "weight decay": match.group(2),
+                            "scheduler": match.group(3),
+                            "gamma": match.group(4),
+                            "inducing points percentage": match.group(5),
+                            "patience": match.group(6),
+                        }
+                        execution_date = re.search(r"^(.*?),", line).group(1)
+            if correct_log_structure:
+                hyperparameters_dataset_algo = {
+                    "algorithm": algorithm,
+                    "version": version,
+                    "dataset": dataset,
+                    "best_hyperparameters": best_hyperparameters,
+                    "execution_date": execution_date,
+                }
+        return hyperparameters_dataset_algo
+
     @staticmethod
     def _filter_list(data):
         """
@@ -203,49 +237,71 @@ class ResultsHandler:
         """
         return list(filter(lambda x: x != "" and x != {}, data))
 
-    def compute_differences(
-        self, base_algorithm: str, results: List[Dict], algorithms: List, err: str
-    ) -> Dict:
-        base_results = results[algorithms.index(base_algorithm)]
-        differences = {}
-        differences_all_algos = []
-        for algorithm in algorithms:
+    def calculate_percent_diff(self, results, base_algorithm):
+        percent_diffs = {}
+        base_result = results[base_algorithm]
+        for algorithm in results.keys():
             if algorithm != base_algorithm:
-                curr_results = results[algorithms.index(algorithm)]
-                diff = {}
-                for metric in base_results:
-                    diff[metric] = {}
-                    for group in base_results[metric].keys():
-                        base_value = base_results[metric][group]
-                        curr_value = curr_results[metric][group]
-                        diff[metric][group] = (
-                            (curr_value - base_value) / base_value * 100
-                        )
-                diff_processed = self._handle_groups(diff, err)
-                df_res_to_plot = self._differences_to_df(diff_processed)
-                df_res_to_plot = df_res_to_plot.assign(algorithm=algorithm)
-                differences_all_algos.append(df_res_to_plot)
-        if not differences_all_algos:
-            raise ValueError("There are no metrics files from gpf variants")
-        df_all_algos_boxplot = pd.concat(differences_all_algos)
-        differences["Difference"] = df_all_algos_boxplot
-        return differences
+                percent_diffs[algorithm] = self.percentage_difference_recur(
+                    base_result, results[algorithm]
+                )
+        percent_diffs_dfs = self.dict_to_df(percent_diffs, base_algorithm)
+        return percent_diffs_dfs
+
+    def percentage_difference_recur(self, base_dict, dict2):
+        """
+        Computes the percentage difference between a base_dict and dict2.
+        Since we are handling error metrics, a positive number means that the
+        dict2 has a higher error than base_dict
+        """
+        diff = {}
+        for key in base_dict:
+            if key in dict2:
+                if type(base_dict[key]) == dict:
+                    diff[key] = self.percentage_difference_recur(base_dict[key], dict2[key])
+                else:
+                    diff[key] = (dict2[key] - base_dict[key]) / (base_dict[key])
+        return diff
 
     @staticmethod
-    def _differences_to_df(diff: Dict) -> pd.DataFrame:
-        rows = []
-        # Iterate over the keys of the data (the groups)
-        for group, values in diff.items():
-            # If the values are a scalar, store them as a single row in the DataFrame
-            if isinstance(values, (int, float)):
-                rows.append({"group": group, "value": values})
-            # If the values are an array, store each value as a separate row in the DataFrame
-            else:
-                for value in values:
-                    rows.append({"group": group, "value": value})
+    def concat_dfs(obj):
+        result = pd.concat([df.assign(algorithm=algorithm) for algorithm, df in obj.items()])
+        return result
 
-        df = pd.DataFrame(rows)
-        return df
+    def dict_to_df(self, data, base_algorithm):
+        dict_df_algo = {}
+        for algorithm in self.algorithms:
+            if algorithm != base_algorithm:
+                data_algo = data[algorithm]
+                df = pd.DataFrame(
+                    columns=["group", "value", "algorithm", "group_element"]
+                )
+                for key, value in data_algo.items():
+                    if type(value) == dict:
+                        for k, v in value.items():
+                            for i, val in enumerate(v):
+                                df = df.append(
+                                    {
+                                        "group": key,
+                                        "value": val,
+                                        "algorithm": algorithm,
+                                        "group_element": k,
+                                    },
+                                    ignore_index=True,
+                                )
+                    else:
+                        for i, val in enumerate(value):
+                            df = df.append(
+                                {
+                                    "group": key,
+                                    "value": val,
+                                    "algorithm": algorithm,
+                                    "group_element": "",
+                                },
+                                ignore_index=True,
+                            )
+                dict_df_algo[algorithm] = df
+        return dict_df_algo
 
     def compute_results_hierarchy(
         self,
@@ -258,13 +314,11 @@ class ResultsHandler:
             algorithm,
             res_measure="mean",
             res_type=res_type,
-            output_type=output_type,
         )
         results_algo_std, algorithm_w_type = self.load_results_algorithm(
             algorithm,
             res_measure="std",
             res_type=res_type,
-            output_type=output_type,
         )
         y_group = {}
         mean_group = {}
@@ -339,32 +393,49 @@ class ResultsHandler:
             group_elements_names,
         )
 
-    def compute_mase(self, results_hierarchy, results_by_group_element, group_elements):
-        mase_by_group = {}
+    def _compute_metric(
+        self, results_hierarchy, results_by_group_element, group_elements, metric="mase"
+    ):
+        res = None
+        metric_by_group = {}
         for group in results_hierarchy[0].keys():
             y_true = results_hierarchy[0][group]
             y_pred = results_hierarchy[1][group]
-            mase = self.mase(
-                y_true=y_true[-self.h :],
-                y_pred=y_pred[-self.h :],
-                y_train=y_true[: self.n - self.h],
-                sp=self.seasonality,
-            )
-            mase_by_group[group] = mase
-        for group, group_ele in group_elements.items():
-            mase_by_element = {}
-            for idx, element in enumerate(group_ele):
-                y_true = results_by_group_element[0][group][:, idx]
-                y_pred = results_by_group_element[1][group][:, idx]
-                mase = self.mase(
+            if metric == "mase":
+                res = self.mase(
                     y_true=y_true[-self.h :],
                     y_pred=y_pred[-self.h :],
                     y_train=y_true[: self.n - self.h],
                     sp=self.seasonality,
                 )
-                mase_by_element[element] = mase
-            mase_by_group[group] = mase_by_element
-        return mase_by_group
+            elif metric == "rmse":
+                res = mean_squared_error(
+                    y_true=y_true[-self.h :],
+                    y_pred=y_pred[-self.h :],
+                    multioutput="raw_values",
+                )
+            metric_by_group[group] = res
+        for group, group_ele in group_elements.items():
+            metric_by_element = {}
+            for idx, element in enumerate(group_ele):
+                y_true = results_by_group_element[0][group][:, idx]
+                y_pred = results_by_group_element[1][group][:, idx]
+                if metric == "mase":
+                    res = self.mase(
+                        y_true=y_true[-self.h :],
+                        y_pred=y_pred[-self.h :],
+                        y_train=y_true[: self.n - self.h],
+                        sp=self.seasonality,
+                    )
+                elif metric == "rmse":
+                    res = mean_squared_error(
+                        y_true=y_true[-self.h :],
+                        y_pred=y_pred[-self.h :],
+                        multioutput="raw_values",
+                    )
+                metric_by_element[element] = res
+            metric_by_group[group] = metric_by_element
+        return metric_by_group
 
     def store_metrics(
         self,
@@ -379,58 +450,5 @@ class ResultsHandler:
         ) as handle:
             pickle.dump(res, handle, pickle.HIGHEST_PROTOCOL)
 
-    def data_to_boxplot(
-        self,
-        err: str,
-        res_measure: str = "",
-        res_type: str = "",
-        output_type: str = "metrics",
-    ) -> pd.DataFrame:
-        """
-        Convert data to a format suitable for creating a boxplot.
 
-        Args:
-            err: The error metric to use for the boxplot.
 
-        Returns:
-            A pandas DataFrame containing the data in a format suitable for creating a
-            boxplot.
-        """
-        dfs = []
-        self._validate_param(res_type, ["fitpred", "pred", ""])
-        self._validate_param(res_measure, ["mean", "std", ""])
-        self._validate_param(output_type, ["results", "metrics", ""])
-        for algorithm in self.algorithms:
-            if self.algorithms_metadata[algorithm]['version']:
-                results, algorithm_w_type = self.load_results_algorithm(
-                    algorithm,
-                    res_measure=res_measure,
-                    res_type=res_type,
-                    output_type=output_type,
-                )
-                if algorithm_w_type:
-                    res_to_plot = self._handle_groups(results, err)
-                    # Create a list of tuples, where each tuple is a group-value pair
-                    data = [
-                        (group, value)
-                        for group in res_to_plot
-                        for value in res_to_plot[group]
-                    ]
-                    df_res_to_plot = pd.DataFrame(data, columns=["group", "value"])
-                    df_res_to_plot = df_res_to_plot.assign(algorithm=algorithm_w_type)
-                    dfs.append(df_res_to_plot)
-        if len(dfs) > 0:
-            df_all_algos_boxplot = pd.concat(dfs)
-        else:
-            df_all_algos_boxplot = None
-        return df_all_algos_boxplot
-
-    @staticmethod
-    def _handle_groups(result_err: Dict, err: str) -> Dict:
-        res_to_plot = {}
-        for group, res in result_err[err].items():
-            # get the individual results to later plot a distribution
-            if "ind" in group:
-                group_name = group.split("_")[0].lower()
-                res_to_plot[group_name] = res
-        return res_to_plot
