@@ -3,7 +3,6 @@ import pandas as pd
 import pickle
 
 from htsexperimentation.compute_results.results_handler import ResultsHandler
-from htsexperimentation.config import RESULTS_PATH
 from htsexperimentation.visualization.plotting import (
     boxplot,
     plot_predictions_hierarchy,
@@ -23,66 +22,63 @@ class TestModel(unittest.TestCase):
                 data[i] = pickle.load(handle)
 
         self.results_prison_gpf = ResultsHandler(
-            path=RESULTS_PATH,
+            path="./results/",
             dataset=self.datasets[0],
-            algorithms=["gpf_exact", "gpf_svg"],
+            algorithms=["gpf_exact", "gpf_svg", "gpf_sparse"],
             groups=data[0],
         )
         self.results_tourism_gpf = ResultsHandler(
-            path=RESULTS_PATH,
+            path="./results/",
             dataset=self.datasets[1],
-            algorithms=["gpf_exact", "gpf_svg"],
+            algorithms=["gpf_exact", "gpf_svg", "gpf_sparse"],
             groups=data[1],
         )
 
         self.results_prison = ResultsHandler(
-            path=RESULTS_PATH,
+            path="./results/",
             dataset=self.datasets[0],
-            algorithms=["mint", "gpf_exact", "deepar", "standard_gp", "ets_bu"],
+            algorithms=["mint", "gpf_exact", "deepar"],
             groups=data[0],
         )
         self.results_tourism = ResultsHandler(
-            path=RESULTS_PATH,
+            path="./results/",
             dataset=self.datasets[1],
-            algorithms=["mint", "gpf_exact", "deepar", "standard_gp", "ets_bu"],
+            algorithms=["mint", "gpf_exact", "deepar"],
             groups=data[1],
         )
 
     def test_results_load(self):
-        res = self.results_prison.load_results_algorithm(algorithm="ets_bu")
+        res = self.results_prison.load_results_algorithm(
+            algorithm="ets_bu",
+            res_type="fitpred",
+            res_measure="mean",
+        )
         self.assertTrue(res)
 
-    def test_create_df_boxplot(self):
-        res = self.results_prison.data_to_boxplot("mase")
-        self.assertTrue(isinstance(res, pd.DataFrame))
-
-    def test_create_boxplot_gpf_variants(self):
-        dataset_res = {}
-        dataset_res[self.datasets[0]] = self.results_prison_gpf.data_to_boxplot("mase")
-        dataset_res[self.datasets[1]] = self.results_tourism_gpf.data_to_boxplot("mase")
-        res = boxplot(datasets_err=dataset_res, err="mase")
-
-    def test_compute_diferences_gpf_variants(self):
-        results, algorithms = self.results_prison_gpf.load_all_results_algo_list(
-            algorithms_list=["gpf_exact", "gpf_svg"], res_type="fit_pred", res_measure="mean"
+    def test_compute_differences_gpf_variants(self):
+        differences = {}
+        results = self.results_prison_gpf.compute_error_metrics(metric="rmse")
+        differences[self.results_prison_gpf.dataset] = self.results_prison_gpf.calculate_percent_diff(
+            base_algorithm="gpf_exact", results=results
         )
-        differences = self.results_prison_gpf.compute_differences(
-            base_algorithm="gpf_exact",
-            results=results,
-            algorithms=algorithms,
-            err="rmse",
+        results = self.results_tourism_gpf.compute_error_metrics(metric="rmse")
+        differences[self.results_tourism_gpf.dataset] = self.results_tourism_gpf.calculate_percent_diff(
+            base_algorithm="gpf_exact", results=results
         )
-        res = boxplot(datasets_err=differences, err="rmse")
+        boxplot(datasets_err=differences, err="rmse")
 
     def test_create_boxplot_all_algorithms(self):
         dataset_res = {}
-        dataset_res[self.datasets[0]] = self.results_prison.data_to_boxplot(
-            "mase", output_type="metrics"
-        )
-        dataset_res[self.datasets[1]] = self.results_tourism.data_to_boxplot(
-            "mase", output_type="metrics"
-        )
-        res = boxplot(datasets_err=dataset_res, err="mase")
+        res_prison = self.results_prison.compute_error_metrics(metric="mase")
+        res_tourism = self.results_tourism.compute_error_metrics(metric="mase")
+
+        res_obj_prison = self.results_prison.dict_to_df(res_prison, "")
+        dataset_res['prison'] = self.results_prison.concat_dfs(res_obj_prison)
+
+        res_obj_tourism = self.results_tourism.dict_to_df(res_tourism, "")
+        dataset_res['tourism'] = self.results_prison.concat_dfs(res_obj_tourism)
+
+        boxplot(datasets_err=dataset_res, err="mase")
 
     def test_compute_mase(self):
         (
@@ -95,10 +91,10 @@ class TestModel(unittest.TestCase):
             *results_by_group_element,
             group_elements,
             self.results_prison.h,
-            'gpf_exact'
+            "gpf_exact",
         )
-        mase_by_group = self.results_prison.compute_mase(
-            results_hierarchy, results_by_group_element, group_elements
+        mase_by_group = self.results_prison._compute_metric(
+            results_hierarchy, results_by_group_element, group_elements, "mase"
         )
         self.assertTrue(
             list(mase_by_group.keys()) == ["bottom", "top", "state", "gender", "legal"]
@@ -110,7 +106,7 @@ class TestModel(unittest.TestCase):
             results_by_group_element,
             group_elements,
         ) = self.results_prison.compute_results_hierarchy(algorithm="mint")
-        mase_by_group = self.results_prison.compute_mase(
-            results_hierarchy, results_by_group_element, group_elements
+        mase_by_group = self.results_prison._compute_metric(
+            results_hierarchy, results_by_group_element, group_elements, "mase"
         )
         plot_mase(mase_by_group)
